@@ -11,8 +11,7 @@
 #' @inheritParams windowDoc
 #' @inheritParams nameDoc
 #' @inheritParams conceptSetDoc
-#' @param inObservation If TRUE only records inside an observation period will
-#' be considered.
+#' @inheritParams softValidationDoc
 #'
 #' @return Cohort table with only those  with the events in the concept list
 #' kept (or those without the event if negate = TRUE)
@@ -39,7 +38,8 @@ requireConceptIntersect <- function(cohort,
                                     targetEndDate = "event_end_date",
                                     inObservation = TRUE,
                                     censorDate = NULL,
-                                    name = tableName(cohort)) {
+                                    name = tableName(cohort),
+                                    .softValidation = TRUE) {
   # checks
   name <- omopgenerics::validateNameArgument(name, validation = "warning")
   cohort <- omopgenerics::validateCohortArgument(cohort)
@@ -49,6 +49,7 @@ requireConceptIntersect <- function(cohort,
   cohortId <- omopgenerics::validateCohortIdArgument({{cohortId}}, cohort, validation = "warning")
   intersections <- validateIntersections(intersections)
   conceptSet <- omopgenerics::validateConceptSetArgument(conceptSet, cdm)
+  omopgenerics::assertLogical(.softValidation)
 
   if (length(cohortId) == 0) {
     cli::cli_inform("Returning entry cohort as `cohortId` is not valid.")
@@ -71,16 +72,6 @@ requireConceptIntersect <- function(cohort,
   upper_limit[is.infinite(upper_limit)] <- 999999L
   upper_limit <- as.integer(upper_limit)
 
-  cols <- unique(
-    c(
-      "cohort_definition_id",
-      "subject_id",
-      "cohort_start_date",
-      "cohort_end_date",
-      indexDate
-    )
-  )
-
   window_start <- window[[1]][1]
   window_end <- window[[1]][2]
 
@@ -90,85 +81,106 @@ requireConceptIntersect <- function(cohort,
 
   if (length(conceptSet) == 0) {
     cli::cli_inform(c("i" = "Empty codelist provided, returning input cohort"))
-  } else {
-    subsetName <- omopgenerics::uniqueTableName()
-    subsetCohort <- cohort |>
-      dplyr::select(dplyr::all_of(.env$cols)) |>
-      PatientProfiles::addConceptIntersectCount(
-        conceptSet = conceptSet,
-        indexDate = indexDate,
-        targetStartDate = targetStartDate,
-        targetEndDate = targetEndDate,
-        window = window,
-        censorDate = censorDate,
-        inObservation = inObservation,
-        nameStyle = "intersect_concept",
-        name = subsetName
-      )
+    return(
+      cohort |>
+        dplyr::compute(
+          name = name, temporary = FALSE,
+          logPrefix = "CohortConstructor_requireConceptIntersect_emptyConcept_"
+        )
+    )
+  }
 
-    subsetCohort <- subsetCohort |>
-      dplyr::mutate(lower_limit = .env$lower_limit,
-                    upper_limit = .env$upper_limit) |>
-      dplyr::filter((
-        .data$intersect_concept >= .data$lower_limit &
-          .data$intersect_concept <= .data$upper_limit
-      ) |
-        (!.data$cohort_definition_id %in% .env$cohortId)
-      ) |>
-      dplyr::select(dplyr::all_of(cols)) |>
-      dplyr::compute(name = subsetName, temporary = FALSE,
-                     logPrefix = "CohortConstructor_requireConceptIntersect_subset_")
+  # temp tables
+  tablePrefix <- omopgenerics::tmpPrefix()
+  tmpNewCohort <- omopgenerics::uniqueTableName(tablePrefix)
+  tmpUnchanged <- omopgenerics::uniqueTableName(tablePrefix)
+  cdm <- filterCohortInternal(cdm, cohort, cohortId, tmpNewCohort, tmpUnchanged)
+  newCohort <- cdm[[tmpNewCohort]]
 
-    # attrition reason
-    if (all(intersections == 0)) {
-      reason <- glue::glue(
-        "Not in concept {names(conceptSet)} between {window_start} & ",
-        "{window_end} days relative to {indexDate}"
-      )
-    } else if (intersections[[1]] != intersections[[2]]) {
-      reason <- glue::glue(
-        "Concept {names(conceptSet)} between {window_start} & ",
-        "{window_end} days relative to {indexDate} between ",
-        "{intersections[[1]]} and {intersections[[2]]}"
-      )
-    } else {
-      reason <- glue::glue(
-        "Concept {names(conceptSet)} between {window_start} & ",
-        "{window_end} days relative to {indexDate} ",
-        "{intersections[[1]]} times"
-      )
-    }
-    if (!is.null(censorDate)) {
-      reason <- glue::glue("{reason}, censoring at {censorDate}")
-    }
-
-    #codelist
-    newCodelist <- getIntersectionCodelist(
-      cohort, cohortId, conceptSetToCohortCodelist(conceptSet)
+  intersectCol <- uniqueColumnName(newCohort)
+  newCohort <- newCohort |>
+    PatientProfiles::addConceptIntersectCount(
+      conceptSet = conceptSet,
+      indexDate = indexDate,
+      targetStartDate = targetStartDate,
+      targetEndDate = targetEndDate,
+      window = window,
+      censorDate = censorDate,
+      inObservation = inObservation,
+      nameStyle = intersectCol,
+      name = tmpNewCohort
     )
 
-    # cohort
-    cohort <- cohort |>
-      dplyr::inner_join(subsetCohort, by = c(cols)) |>
-      dplyr::compute(name = name, temporary = FALSE,
-                     logPrefix = "CohortConstructor_requireConceptIntersect_join_") |>
-      omopgenerics::newCohortTable(
-        .softValidation = TRUE, cohortCodelistRef = newCodelist
-      ) |>
-      omopgenerics::recordCohortAttrition(reason = reason, cohortId = cohortId)
+  newCohort <- newCohort |>
+    dplyr::filter(
+      .data[[intersectCol]] >= .env$lower_limit &
+        .data[[intersectCol]] <= .env$upper_limit
+    ) |>
+    dplyr::select(!dplyr::all_of(intersectCol)) |>
+    dplyr::compute(name = tmpNewCohort, temporary = FALSE,
+                   logPrefix = "CohortConstructor_requireConceptIntersect_subset_")
 
-    omopgenerics::dropTable(cdm = cdm, name = subsetName)
+  # attrition reason
+  if (all(intersections == 0)) {
+    reason <- glue::glue(
+      "Not in concept {names(conceptSet)} between {window_start} & ",
+      "{window_end} days relative to {indexDate}"
+    )
+  } else if (intersections[[1]] != intersections[[2]]) {
+    reason <- glue::glue(
+      "Concept {names(conceptSet)} between {window_start} & ",
+      "{window_end} days relative to {indexDate} between ",
+      "{intersections[[1]]} and {intersections[[2]]}"
+    )
+  } else {
+    reason <- glue::glue(
+      "Concept {names(conceptSet)} between {window_start} & ",
+      "{window_end} days relative to {indexDate} ",
+      "{intersections[[1]]} times"
+    )
   }
+  if (!is.null(censorDate)) {
+    reason <- glue::glue("{reason}, censoring at {censorDate}")
+  }
+
+  # codelist
+  newCodelist <- getIntersectionCodelist(
+    cohort, cohortId, conceptSetToCohortCodelist(conceptSet)
+  )
+
+  if (isTRUE(needsIdFilter(cohort, cohortId))) {
+    newCohort <- newCohort |>
+      # join non modified cohorts
+      dplyr::union_all(
+        cdm[[tmpUnchanged]] |>
+          dplyr::select(dplyr::all_of(colnames(newCohort)))
+      ) |>
+      dplyr::compute(name = tmpNewCohort, temporary = FALSE,
+                     logPrefix = "CohortConstructor_requireConceptIntersect_union_")
+  }
+
+  # cohort
+  newCohort <- newCohort |>
+    dplyr::compute(
+      name = name, temporary = FALSE,
+      logPrefix = "CohortConstructor_requireConceptIntersect_join_"
+    ) |>
+    omopgenerics::newCohortTable(
+      .softValidation = .softValidation, cohortCodelistRef = newCodelist
+    ) |>
+    omopgenerics::recordCohortAttrition(reason = reason, cohortId = cohortId)
+
+  omopgenerics::dropSourceTable(cdm = cdm, name = dplyr::starts_with(tablePrefix))
 
   useIndexes <- getOption("CohortConstructor.use_indexes")
   if (!isFALSE(useIndexes)) {
     addIndex(
-      cohort = cohort,
+      cohort = newCohort,
       cols = c("subject_id", "cohort_start_date")
     )
   }
 
-  return(cohort)
+  return(newCohort)
 }
 
 getIntersectionCodelist <- function(cohort, cohortId, codelist) {
@@ -176,12 +188,15 @@ getIntersectionCodelist <- function(cohort, cohortId, codelist) {
   intersectCodelist <- lapply(
     as.list(cohortId),
     function(x, tab = codelist) {
-      tab |> dplyr::mutate(cohort_definition_id = .env$x)
+      tab |>
+        dplyr::mutate(cohort_definition_id = .env$x) |>
+        dplyr::select(!dplyr::any_of(c("type", "codelist_type")))
     }) |>
     dplyr::bind_rows() |>
     dplyr::mutate(type = .env$criteria)
   newCodelist <- attr(cohort, "cohort_codelist") |>
     dplyr::collect() |>
+    dplyr::rename(type = dplyr::any_of("codelist_type")) |>
     dplyr::union(intersectCodelist) |>
     dplyr::arrange(.data$cohort_definition_id)
   return(newCodelist)
