@@ -11,6 +11,7 @@
 #' which the index date must have been observed.
 #' @param indexDate Name of the column in the cohort that contains the date of
 #' interest.
+#' @inheritParams softValidationDoc
 #'
 #' @return The cohort table with any cohort entries outside of the date range
 #' dropped
@@ -29,7 +30,8 @@ requireInDateRange <- function(cohort,
                                dateRange,
                                cohortId = NULL,
                                indexDate = "cohort_start_date",
-                               name = tableName(cohort)) {
+                               name = tableName(cohort),
+                               .softValidation = TRUE) {
   # checks
   name <- omopgenerics::validateNameArgument(name, validation = "warning")
   cohort <- omopgenerics::validateCohortArgument(cohort)
@@ -37,6 +39,7 @@ requireInDateRange <- function(cohort,
   cdm <- omopgenerics::validateCdmArgument(omopgenerics::cdmReference(cohort))
   cohortId <- omopgenerics::validateCohortIdArgument({{cohortId}}, cohort, validation = "warning")
   dateRange <- validateDateRange(dateRange)
+  omopgenerics::assertLogical(.softValidation)
 
   if (length(cohortId) == 0) {
     cli::cli_inform("Returning entry cohort as `cohortId` is not valid.")
@@ -46,31 +49,45 @@ requireInDateRange <- function(cohort,
     return(cdm[[name]])
   }
 
+  # filters
+  filt1 <- glue::glue(".data[[indexDate]] >= as.Date('{dateRange[1]}')")
+  filt2 <- glue::glue(".data[[indexDate]] <= as.Date('{dateRange[2]}')")
+  if (isTRUE(needsIdFilter(cohort, cohortId))) {
+    filt1 <- glue::glue("{filt1} | (!.data$cohort_definition_id %in% cohortId)")
+    filt2 <- glue::glue("{filt2} | (!.data$cohort_definition_id %in% cohortId)")
+  }
+  filt1 <- rlang::parse_exprs(glue::glue(filt1))
+  filt2 <- rlang::parse_exprs(glue::glue(filt2))
+
   # requirement
   if (!is.na(dateRange[1])) {
     cohort <- cohort |>
-      dplyr::filter(.data[[indexDate]] >= !!dateRange[1] |
-                      (!.data$cohort_definition_id %in% cohortId)) |>
-      dplyr::compute(name = name, temporary = FALSE,
-                     logPrefix = "CohortConstructor_requireInDateRange_dateRange1_") |>
+      dplyr::filter(!!!filt1) |>
+      dplyr::compute(
+        name = name, temporary = FALSE,
+        logPrefix = "CohortConstructor_requireInDateRange_dateRange1_"
+      ) |>
       omopgenerics::recordCohortAttrition(reason = "{indexDate} after {dateRange[1]}",
                                           cohortId = cohortId)
   }
 
   if (!is.na(dateRange[2])) {
     cohort <- cohort |>
-      dplyr::filter(.data[[indexDate]] <= !!dateRange[2] |
-                      (!.data$cohort_definition_id %in% cohortId)) |>
-      dplyr::compute(name = name, temporary = FALSE,
-                     logPrefix = "CohortConstructor_requireInDateRange_dateRange2_") |>
+      dplyr::filter(!!!filt2) |>
+      dplyr::compute(
+        name = name, temporary = FALSE,
+        logPrefix = "CohortConstructor_requireInDateRange_dateRange2_"
+      ) |>
       omopgenerics::recordCohortAttrition(reason = "{indexDate} before {dateRange[2]}",
                                           cohortId = cohortId)
   }
 
   cohort <- cohort |>
-    dplyr::compute(name = name, temporary = FALSE,
-                   logPrefix = "CohortConstructor_requireInDateRange_newCohort_") |>
-    omopgenerics::newCohortTable(.softValidation = TRUE)
+    dplyr::compute(
+      name = name, temporary = FALSE,
+      logPrefix = "CohortConstructor_requireInDateRange_newCohort_"
+    ) |>
+    omopgenerics::newCohortTable(.softValidation = .softValidation)
 
   useIndexes <- getOption("CohortConstructor.use_indexes")
   if (!isFALSE(useIndexes)) {
@@ -96,6 +113,7 @@ requireInDateRange <- function(cohort,
 #'  have been observed.
 #' @param startDate Variable with earliest date.
 #' @param endDate Variable with latest date.
+#' @inheritParams softValidationDoc
 #'
 #' @return The cohort table with record timings updated to only be within the
 #' date range. Any records with all time outside of the range will have
@@ -118,7 +136,8 @@ trimToDateRange <- function(cohort,
                             cohortId = NULL,
                             startDate = "cohort_start_date",
                             endDate = "cohort_end_date",
-                            name = tableName(cohort)) {
+                            name = tableName(cohort),
+                            .softValidation = FALSE) {
   # checks
   name <- omopgenerics::validateNameArgument(name, validation = "warning")
   cohort <- omopgenerics::validateCohortArgument(cohort)
@@ -127,93 +146,116 @@ trimToDateRange <- function(cohort,
   cdm <- omopgenerics::validateCdmArgument(omopgenerics::cdmReference(cohort))
   cohortId <- omopgenerics::validateCohortIdArgument({{cohortId}}, cohort, validation = "warning")
   dateRange <- validateDateRange(dateRange)
+  omopgenerics::assertLogical(.softValidation)
 
   if (length(cohortId) == 0) {
     cli::cli_inform("Returning entry cohort as `cohortId` is not valid.")
     # return entry cohort as cohortId is used to modify not subset
-    cdm[[name]] <- cohort |> dplyr::compute(name = name, temporary = FALSE,
-                                            logPrefix = "CohortConstructor_trimToDateRange_entry_")
+    cdm[[name]] <- cohort |>
+      dplyr::compute(
+        name = name, temporary = FALSE,
+        logPrefix = "CohortConstructor_trimToDateRange_entry_"
+      )
     return(cdm[[name]])
   }
 
+  # temp tables
+  tablePrefix <- omopgenerics::tmpPrefix()
+  tmpNewCohort <- omopgenerics::uniqueTableName(tablePrefix)
+  tmpUnchanged <- omopgenerics::uniqueTableName(tablePrefix)
+  cdm <- filterCohortInternal(cdm, cohort, cohortId, tmpNewCohort, tmpUnchanged)
+  newCohort <- cdm[[tmpNewCohort]]
+
   # trim start
   if (!is.na(dateRange[1])) {
-    cohort <- cohort |>
+    newCohort <- newCohort |>
       trimStartDate(
-        requirementIds = cohortId,
         startDate = startDate,
         endDate = endDate,
         minDate = dateRange[1]
       ) |>
-      dplyr::compute(name = name, temporary = FALSE,
-                     logPrefix = "CohortConstructor_trimToDateRange_start_") |>
-      omopgenerics::recordCohortAttrition(reason = "{startDate} trimmed >= {dateRange[1]}",
-                                          cohortId = cohortId)
+      dplyr::compute(
+        name = tmpNewCohort, temporary = FALSE,
+        logPrefix = "CohortConstructor_trimToDateRange_start_"
+      ) |>
+      omopgenerics::recordCohortAttrition(
+        reason = "{startDate} trimmed >= {dateRange[1]}", cohortId = cohortId
+      )
   }
 
   # trim end
   if (!is.na(dateRange[2])) {
-    cohort <- cohort |>
+    newCohort <- newCohort |>
       trimEndDate(
-        requirementIds = cohortId,
         startDate = startDate,
         endDate = endDate,
         maxDate = dateRange[2]
       ) |>
-      dplyr::compute(name = name, temporary = FALSE,
-                     logPrefix = "CohortConstructor_trimToDateRange_end_") |>
-      omopgenerics::recordCohortAttrition(reason = "{endDate} trimmed <= {dateRange[2]}",
-                                          cohortId = cohortId)
+      dplyr::compute(
+        name = tmpNewCohort, temporary = FALSE,
+        logPrefix = "CohortConstructor_trimToDateRange_end_"
+      ) |>
+      omopgenerics::recordCohortAttrition(
+        reason = "{endDate} trimmed <= {dateRange[2]}", cohortId = cohortId
+      )
   }
 
-  cohort <- cohort |>
-    dplyr::compute(name = name, temporary = FALSE,
-                   logPrefix = "CohortConstructor_trimToDateRange_newCohort_") |>
-    omopgenerics::newCohortTable(.softValidation = FALSE)
+  if (isTRUE(needsIdFilter(cohort, cohortId))) {
+    newCohort <- newCohort |>
+      # join non modified cohorts
+      dplyr::union_all(cdm[[tmpUnchanged]]) |>
+      dplyr::compute(
+        name = tmpNewCohort, temporary = FALSE,
+        logPrefix = "CohortConstructor_requireDateRange_union_"
+      )
+  }
+
+  newCohort <- newCohort |>
+    dplyr::compute(
+      name = name, temporary = FALSE,
+      logPrefix = "CohortConstructor_trimToDateRange_newCohort_"
+    ) |>
+    omopgenerics::newCohortTable(.softValidation = .softValidation)
+
+  omopgenerics::dropSourceTable(cdm = cdm, name = dplyr::starts_with(tablePrefix))
 
   useIndexes <- getOption("CohortConstructor.use_indexes")
   if (!isFALSE(useIndexes)) {
     addIndex(
-      cohort = cohort,
+      cohort = newCohort,
       cols = c("subject_id", "cohort_start_date")
     )
   }
 
-  return(cohort)
+  return(newCohort)
 }
 
 trimStartDate <- function(cohort,
                           startDate,
                           endDate,
-                          minDate,
-                          requirementIds) {
+                          minDate) {
   cohort <- cohort |>
     dplyr::mutate(
       !!startDate := dplyr::if_else(
-        .data[[startDate]] <= !!minDate &
-          .data$cohort_definition_id %in% .env$requirementIds,
+        .data[[startDate]] <= !!minDate,
         as.Date(minDate),
         .data[[startDate]]
       )
     ) |>
-    dplyr::filter(.data[[startDate]] <= .data[[endDate]] |
-                    (!.data$cohort_definition_id %in% .env$requirementIds))
+    dplyr::filter(.data[[startDate]] <= .data[[endDate]])
   return(cohort)
 }
 
 trimEndDate <- function(cohort,
                         startDate,
                         endDate,
-                        maxDate,
-                        requirementIds) {
+                        maxDate) {
   cohort <- cohort |>
     dplyr::mutate(!!endDate := dplyr::if_else(
-      .data[[endDate]] >= !!maxDate &
-        (.data$cohort_definition_id %in% .env$requirementIds),
+      .data[[endDate]] >= !!maxDate,
       as.Date(maxDate),
       .data[[endDate]]
     )) |>
-    dplyr::filter(.data[[startDate]] <= .data[[endDate]] |
-                    (!.data$cohort_definition_id %in% requirementIds))
+    dplyr::filter(.data[[startDate]] <= .data[[endDate]])
   return(cohort)
 }
