@@ -18,7 +18,6 @@ test_that("test restrict to first entry works", {
 
   expect_true(all(cdm$cohort2 |> CohortConstructor::requireIsFirstEntry() |>
                     dplyr::pull(subject_id) == c(1:3, 1:3)))
-
 })
 
 test_that("requireIsFirstEntry, cohortIds & name arguments", {
@@ -57,6 +56,15 @@ test_that("requireIsFirstEntry, cohortIds & name arguments", {
         "Initial qualifying events", "Initial qualifying events")
   ))
 
+  cdm$new_cohort_2 <- CohortConstructor::requireIsEntry(
+    entryRange = 1,
+    cohort = cdm$cohort,
+    cohortId = 1,
+    name = "new_cohort_2")
+
+  expect_identical(
+  nrow(cdm$new_cohort |> dplyr::collect()),
+  nrow(cdm$new_cohort_2 |> dplyr::collect()))
 })
 
 test_that("errors", {
@@ -213,6 +221,9 @@ test_that("requireEntry", {
    dplyr::pull("cohort_start_date")),
    as.Date(c("2010-06-01","2010-07-01")),
    ignore_attr = TRUE)
+
+ expect_true(sum(grepl("og", omopgenerics::listSourceTables(cdm))) == 0)
+ PatientProfiles::mockDisconnect(cdm)
 })
 
 test_that("test indexes - postgres", {
@@ -228,8 +239,8 @@ test_that("test indexes - postgres", {
   cdm <- CDMConnector::cdmFromCon(
     con = db,
     cdmSchema = Sys.getenv("CDM5_POSTGRESQL_CDM_SCHEMA"),
-    writeSchema = c(schema =  Sys.getenv("CDM5_POSTGRESQL_SCRATCH_SCHEMA"),
-                     prefix = "cc_"),
+    writeSchema = Sys.getenv("CDM5_POSTGRESQL_SCRATCH_SCHEMA"),
+    writePrefix = "cc_",
     achillesSchema = Sys.getenv("CDM5_POSTGRESQL_CDM_SCHEMA")
   )
 
@@ -265,6 +276,7 @@ test_that("test indexes - postgres", {
       "CREATE INDEX cc_my_cohort_subject_id_cohort_start_date_idx ON public.cc_my_cohort USING btree (subject_id, cohort_start_date)"
   )
 
-  omopgenerics::dropTable(cdm = cdm, name = dplyr::starts_with("my_cohort"))
+  expect_true(sum(grepl("og", omopgenerics::listSourceTables(cdm))) == 0)
+  omopgenerics::dropSourceTable(cdm = cdm, name = dplyr::starts_with("my_cohort"))
   CDMConnector::cdmDisconnect(cdm = cdm)
 })
